#ifndef PYTHONIC_INCLUDE_ARRAY_TYPECODES_HPP
#define PYTHONIC_INCLUDE_ARRAY_TYPECODES_HPP

#include "pythonic/types/str.hpp"

PYTHONIC_NS_BEGIN

namespace array
{
  static types::str const typecodes("bBuhHiIlLqQfd");
}
PYTHONIC_NS_END

#endif
