#ifndef PYTHONIC_DISPATCH_TOLIST_HPP
#define PYTHONIC_DISPATCH_TOLIST_HPP

#include "pythonic/include/__dispatch__/tolist.hpp"
#include "pythonic/numpy/ndarray/tolist.hpp"
#include "pythonic/types/array.hpp"

#endif
